from abc import ABC, abstractmethod
from selenium import webdriver

from stores.data_store_interface import IDataStore


class IDataCollectionAdapter(ABC):
    """
    数据收集适配器抽象类
    """

    @property
    @abstractmethod
    def url(self):
        """
        驱动器初始打开的URL

        :return:  驱动器初始打开的URL
        :rtype: str
        """
        pass

    @property
    @abstractmethod
    def driver(self):
        """
        创建的驱动器

        :return:  驱动器
        :rtype: webdriver
        """
        pass

    @abstractmethod
    def create_driver(self) -> webdriver:
        """
        根据URL创建一个浏览器驱动器

        :return: 创建的驱动器
        :rtype: webdriver
        :raises Exception: 如果创建驱动器失败，应抛出异常
        """
        try:
            # 实现创建驱动器的逻辑
            pass
        except Exception as e:
            raise Exception(f"创建驱动器失败: {e}")

    @abstractmethod
    def login(self, driver, username: str, password: str) -> None:
        """
        如果需要登录，则登录后跳转到搜索页面（不自动跳转的需要手动执行）

        :param driver: 浏览器驱动器实例
        :param username: 用户名
        :type username: str
        :param password: 密码
        :type password: str
        :raises Exception: 如果登录失败，应抛出异常
        """
        try:
            # 实现登录逻辑
            pass
        except Exception as e:
            raise Exception(f"登录失败: {e}")

    @abstractmethod
    def search(self, driver, keyword: str) -> list:
        """
        根据关键字搜索，返回搜索结果列表

        :param driver: 浏览器驱动器实例
        :param keyword: 搜索关键字
        :type keyword: str
        :return: 搜索结果列表
        :rtype: list
        :raises Exception: 如果搜索失败，应抛出异常
        """
        try:
            results = []
            # 实现搜索逻辑
            return results if results else []
        except Exception as e:
            raise Exception(f"搜索失败: {e}")

    @abstractmethod
    def collect(self, driver, items: list, store: IDataStore) -> list:
        """
        处理搜索结果列表，返回处理后的数据列表

        :param driver: 浏览器驱动器实例
        :param items: 搜索结果列表
        :param store: 数据储存库
        :type items: list
        :return: 处理后的数据列表
        :rtype: list
        :raises Exception: 如果处理失败，应抛出异常
        """
        try:
            processed_items = []
            if items:
                # 实现处理逻辑
                pass
            return processed_items
        except Exception as e:
            raise Exception(f"处理失败: {e}")

    @abstractmethod
    def teardown(self, driver) -> None:
        """
        关闭浏览器驱动器

        :param driver: 浏览器驱动器实例
        :raises Exception: 如果关闭驱动器失败，应抛出异常
        """
        try:
            if driver:
                driver.quit()
        except Exception as e:
            raise Exception(f"关闭驱动器失败: {e}")
