import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.mime.base import MIMEBase
from email import encoders
import os

from utils.config_helper import ConfigHelper
from utils.logger_helper import LoggerHelper


def _attach_file(msg: MIMEMultipart, attachment_path: str):
    if not os.path.isfile(attachment_path):
        raise FileNotFoundError(
            f"The file {attachment_path} does not exist.")

    with open(attachment_path, "rb") as attachment:
        part = MIMEBase('application', 'octet-stream')
        part.set_payload(attachment.read())
        encoders.encode_base64(part)
        part.add_header(
            'Content-Disposition',
            f"attachment; filename= {os.path.basename(attachment_path)}")
        msg.attach(part)


class EmailHelper:

    logger = LoggerHelper.get_logger()
    config = ConfigHelper()

    def __init__(self):
        self.smtp_server = self.config.get("email.smtp_server")
        self.port = self.config.get("email.smtp_port")
        self.username = self.config.get("email.smtp_user")
        self.password = self.config.get("email.smtp_password")
        self.from_email = self.config.get("email.from_email")
        # print(
        #     f"server:{self.smtp_server},port:{self.port},username:{self.username},password:{self.password},from_email:{self.from_email}"
        # )

    def send_email(self,
                   to_addr: str,
                   subject: str,
                   body: str,
                   body_is_html: bool = True,
                   attachment_path: str = None):
        msg = MIMEMultipart()
        msg['From'] = self.from_email
        msg['To'] = ', '.join(to_addr.split(','))
        msg['Subject'] = subject

        # 根据 body_is_html 参数设置 MIMEText 类型
        if body_is_html:
            msg.attach(MIMEText(body, 'html', 'utf-8'))
        else:
            msg.attach(MIMEText(body, 'plain', 'utf-8'))

        if attachment_path:
            _attach_file(msg, attachment_path)

        try:
            with smtplib.SMTP_SSL(self.smtp_server, timeout=10) as server:
                # server.starttls()
                server.login(self.username, self.password)
                # 将 to_addr 字符串通过 split(',') 分割成列表，传递给 sendmail
                server.sendmail(self.from_email, to_addr.split(','),
                                msg.as_string())
            self.logger.info(f"邮件发送成功：{to_addr}")
            return True
        except Exception as e:
            self.logger.error(f"邮件发送失败：{to_addr} {e}")
            return False
